import { type TResolver } from './Resolver';
import { type DeepMergeTwoTypes, type StringDoentContain, type Util_FlatType } from './utils/Types';
/**
 * StringParser version 1.0.0
 * All string parsing function should be in this class
 * this simplifies keeping things alinged in the public and private repo.
 * ---
 * At some point this will become an npm library
 */
export type ResolverSegmentVariable<T extends string> = T extends `{${infer Name}=${infer Value}}` ? {
    [K in Name]: Value;
} : {};
export type TemplateSegmentVariable<T extends string> = T extends `{${infer Name}}` ? {
    [K in Name]: string;
} : {};
export type TEnsureTemplateVariable<T extends string> = T extends `{${infer Name}}` ? StringDoentContain<Name, '='> extends true ? `{${Name}}` : never : never;
export type SegmentVariable<T extends string> = ResolverSegmentVariable<T> extends Record<string, never> ? TemplateSegmentVariable<T> : ResolverSegmentVariable<T>;
export type TemplateSegments<T extends string> = T extends `${infer P1}.${infer Rest}` ? [P1, ...TemplateSegments<Rest>] : [T];
export type ResolverSegmentsVariables<T extends string[]> = Util_FlatType<T extends [infer A, ...infer Rest] ? ((A extends string ? SegmentVariable<A> : {}) & (Rest extends string[] ? ResolverSegmentsVariables<Rest> : {})) : {}>;
export type ResolverVariables<T extends string> = ResolverSegmentsVariables<TemplateSegments<T>>;
export type StringValueVariable<T extends string> = Util_FlatType<`${T} ` extends `${string}$${infer Name} ${infer Rest}` ? (DeepMergeTwoTypes<({
    [K in Name]: string;
}), (Rest extends '' ? {} : StringValueVariable<Rest>)>) : {}>;
export type TVariable = {
    name: string;
    value: string;
};
export declare enum EPathToNodeRelation {
    shadow = "shadow",
    sibling = "sibling",
    parent = "parent",
    child = "child",
    unrelated = "unrelated",
    parentShadow = "parentShadow",
    self = "self"
}
export type TResolvedPathPart = {
    segmentPath: string;
    index: number;
    variable?: TVariable;
};
export type TGraphNode<T = unknown> = {
    inferredFrom: string;
    segmentPath: string;
    value?: T;
    children: TGraphNode<T>[];
};
export type TShadowNodeMap<T = unknown> = Record<string, Array<{
    resolvedPath: string;
    node: TGraphNode<T>;
}>>;
export type TReduceGraphNodeResult<T, X> = {
    [K: string]: TReduceGraphNodeResult<T, X> | X;
};
declare class GraphBase {
    readonly DEFAULT_FILE_SLUG = "default";
    readonly DEFAULT_STRING_SLUG = "default";
    readonly GRAPH_NODE_ROOT_SEGMENT_PATH = "__root";
    readonly ANY_SEGMENT = "___";
    constructor();
    readonly selectGraphPath: ({ path, node, }: {
        path: string;
        node: Record<string, any>;
    }) => any;
    readonly reduceGraphNode: <T, X>({ nodes, nodeToSegment, nodeToChildren, nodeToValue, segmentToKey, }: {
        nodes: T[];
        nodeToSegment: (s: T) => string;
        nodeToChildren: (s: T) => T[];
        nodeToValue: (s: T) => X;
        segmentToKey?: (s: string) => string;
    }) => TReduceGraphNodeResult<T, X>;
    readonly createGraphNodeRoot: <T>(children?: TGraphNode<T>[]) => TGraphNode<T>;
    readonly mergeInternalGraphNodesToCommonRoot: <T>(values: Record<string, T>) => TGraphNode<T>;
    readonly makeInternalGraphNode: <T>(resolvedPath: string, value: T) => TGraphNode<T>;
    readonly mergeInternalGraphNodes: <T>(_a: TGraphNode<T>, _b: TGraphNode<T>) => TGraphNode<T>[];
    readonly graphNodeToVariables: <T>(node: TGraphNode<T>) => Record<string, Record<string, string>>;
    readonly flattenToValues: <T>(node: TGraphNode<T>) => T[];
    readonly resolveFromPathToNode: <T, R>(params: {
        nodes: Record<string, T>;
        nodeToValue: (t: TGraphNode<T>) => R;
        segmentToKey?: (s: string) => string;
    }) => TReduceGraphNodeResult<TGraphNode<T>, R>;
    readonly resetGraphNode: <T = unknown>(graphNode: TGraphNode) => TGraphNode<T>;
}
export declare const Graph: GraphBase;
export {};
