import { type ResolverVariables } from './Graph';
export type TResolver = string;
export declare enum EPathToNodeRelation {
    shadow = "shadow",
    sibling = "sibling",
    parent = "parent",
    child = "child",
    unrelated = "unrelated",
    parentShadow = "parentShadow",
    self = "self"
}
declare class ResolverBase {
    constructor();
    readonly split: (resolver: string) => string[];
    /**
     * Given a partially resolved template like
     *    stringifyr.{locale=it}.blog.{post}
     *
     * Return a fully unresolved template
     *    stringifyr.{locale}.blog.{post}
     */
    readonly toTemplate: (resolver: string) => string;
    /**
     * Returns all variants of a resolver
     *
     * Example:
     *  Passing in "app.{lang=en}.hello.{ok=p1}.blog" will return
     *    [
     *      "app.{lang=en}.hello.{ok=p1}.blog",
     *      "app.{lang}.hello.{ok=p1}.blog",
     *      "app.{lang=en}.hello.{ok}.blog",
     *      "app.{lang}.hello.{ok}.blog",
     *    ]
     */
    readonly variants: (resolver: string) => Set<string>;
    /**
     * Given a partial resolver like:
     *    "stringifyr.{locale=it}.blog.{post}"
     *
     * Return a map of all the variables and their maybe existing values.
     *    {
     *      locale: 'it',
     *      post: undefined,
     *    }
     */
    readonly variables: (resolver: string) => Record<string, string>;
    readonly _dot_memoized: (resolver: string) => object;
    readonly graph: <T>(resolvers: Record<string, T>) => any;
    readonly lastSegment: (resolver: string) => string;
    readonly parent: (resolver: string, level?: number) => string;
    readonly isParent: (resolver: string, parent: string) => boolean;
    readonly isChild: (resolver: string, child: string) => boolean;
    readonly filter: (resolver: string, filter: string) => boolean;
    /**
     * Returns all parents of a resolver including self
     * "a.b.c" => ["a", "a.b", "a.b.c"]
     */
    readonly parents: (resolver: string) => string[];
    readonly compareResolvers: (compare: string, to: string) => EPathToNodeRelation;
    readonly replaceResolver: (previousResolver: string, nextResolver: string, targets: string[]) => (string | null)[];
    readonly resolve: <T extends string>(templateOrResolver: T, variables: ResolverVariables<T>) => string;
    /**
     * Given a partially resolved template like
     *    "stringifyr.{locale=it}.blog.{post}"
     * and variables like
     *    { post: 'en' }
     *
     * Return the resolved version by applying variables
     *    "stringifyr.{locale=it}.blog.{post=en}"
     *
     * Variables that are already in the template have priority
     * over variables that are in the variables map
     */
    readonly resolveUnchecked: <T extends string>(template: T, variables: ResolverVariables<T>) => string;
}
export declare const Resolver: ResolverBase;
export {};
