import { type Object, type String } from 'ts-toolbelt';
export type NoExtraPropertiesV2<T, Shape = T, Depth extends number = 1> = T & Shape & ExtractExcessProperties<T, Shape, Depth>;
export type ExtractExcessProperties<T, Shape, Depth extends number = 1> = {
    done: unknown;
    recur: {
        [K in keyof T]: K extends keyof Required<Shape> ? T[K] extends object ? ExtractExcessProperties<T[K], Required<Shape>[K], [
            -1,
            0,
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9,
            10,
            11,
            12,
            13,
            14,
            15,
            16,
            17,
            18,
            19,
            20
        ][Depth]> : unknown : never;
    } extends infer I ? {
        [K in keyof I as unknown extends I[K] ? never : K]-?: I[K];
    } : never;
}[Depth extends 0 ? 'done' : 'recur'];
export type Impossible<K extends keyof any> = {
    [P in K]: never;
};
export type NoExtraProperties<T, U extends T = T> = U & Impossible<Exclude<keyof U, keyof T>>;
export type NoUndefinedField<T> = {
    [P in keyof T]-?: NoUndefinedField<NonNullable<T[P]>>;
};
export type FirstParameter<Func extends (...args: any) => any> = Parameters<Func>[0];
export type SecondParameter<Func extends (...args: any) => any> = Parameters<Func>[1];
export type OmitFirstParameter<F> = F extends (x: any, ...args: infer P) => infer R ? (...args: P) => R : never;
export type TUnSubscriber = {
    unsubscribe: (reason?: string) => void;
};
export type Without<T, U> = {
    [P in Exclude<keyof T, keyof U>]?: never;
};
export type EmptyObject = {
    [k: string]: never;
};
export type NonNullableShallow<T> = {
    [P in keyof T]: Required<NonNullable<T[P]>>;
};
type NonFunctionPropertyNames<T> = {
    [K in keyof T]: T[K] extends Function ? never : K;
}[keyof T];
export type NonFunctionProperties<T> = Pick<T, NonFunctionPropertyNames<T>>;
export type IsNeverType<T> = [T] extends [never] ? true : false;
export type IsAnyType<T> = [T] extends [any] ? true : false;
export type IsUnknownType<T> = [T] extends [unknown] ? true : false;
export type AbstractInstanceType<T> = T extends {
    prototype: infer U;
} ? U : never;
export type Class<T> = new (...args: any[]) => T;
export type XOR<T, U> = (T | U) extends object ? (Without<T, U> & U) | (Without<U, T> & T) : T | U;
export type Writeable<T> = {
    -readonly [P in keyof T]: Writeable<T[P]>;
};
export type Mutable<Type> = {
    -readonly [Key in keyof Type]: Type[Key];
};
/**
 * Common properties of two objects
 */
type MappedC<A, B> = {
    [K in keyof A & keyof B]: A[K] extends B[K] ? K : never;
};
type Kc<A, B> = MappedC<A, B>[keyof A & keyof B];
export type CommonProperties<A, B> = {
    [K in Kc<A, B>]: A[K];
};
export type TFunc = (...any: any) => any;
/**
 * Obtain the parameters of a function type in a tuple
 */
export type CurryFuncParams<T extends (...args: any[]) => any> = T extends (...args: infer P) => any ? P : never;
/**
 * Obtain the return type of a function type
 */
export type CurryFuncReturnType<T extends (...args: any[]) => any> = T extends (...args: any[]) => infer R ? R : any;
export type CurryFunc<T extends (...args: any[]) => any> = (...funcArgs: CurryFuncParams<T>) => CurryFuncReturnType<T>;
export type AsyncReturnType<T extends (...args: any) => any> = T extends (...args: any) => Promise<infer U> ? U : T extends (...args: any) => infer U ? U : any;
export type MaybePromise<AwaitedT> = AwaitedT extends Promise<infer T> ? T : AwaitedT;
export type ObjOfType<P extends string, Z, X extends {
    [k1 in P]: X[k1] extends Z ? X[k1] : never;
}> = {
    [k1 in P]: X[k1];
};
export type DeepPartial<T> = {
    [P in keyof T]?: T[P] extends Array<infer I> ? Array<DeepPartial<I>> : DeepPartial<T[P]>;
};
export type OmitDistributive<T, K extends PropertyKey> = T extends any ? (T extends object ? Id<OmitRecursively<T, K>> : T) : never;
export type Id<T> = {} & {
    [P in keyof T]: T[P];
};
export type OmitRecursively<T extends any, K extends PropertyKey> = Omit<{
    [P in keyof T]: OmitDistributive<T[P], K>;
}, K>;
export type FixKeys<K extends string, V> = {
    [k in K]: V;
};
/**
 * Get all of the keys to which U can be assigned.
 */
type OnlyKeys<T, U> = {
    [K in keyof T]: U extends T[K] ? K : never;
}[keyof T];
/**
 * Get the interface containing only properties to which U can be assigned.
 */
type OnlyUndefined<T> = {
    [K in OnlyKeys<T, undefined>]: T[K];
};
export type ExpectOfType<T, Expect, Then> = T extends Expect ? Then : never;
/**
 * Get all of the keys except those to which U can be assigned.
 */
export type ExcludeKeys<T, U> = {
    [K in keyof T]: U extends T[K] ? never : K;
}[keyof T];
/**
 * Get the interface containing no properties to which U can be assigned.
 */
type ExcludeUndefined<T> = {
    [K in ExcludeKeys<T, undefined>]: T[K];
};
/**
 * Get the interface where all properties are optional.
 */
export type Optional<T> = {
    [K in keyof T]?: T[K];
};
/**
 * Get the interface where properties that can be assigned undefined are
 * also optional.
 */
export type UndefinedOptional<T> = ExcludeUndefined<T> & Optional<OnlyUndefined<T>>;
export type RequireAtLeastOne<T, Keys extends keyof T = keyof T> = Pick<T, Exclude<keyof T, Keys>> & {
    [K in Keys]-?: Required<Pick<T, K>> & Partial<Pick<T, Exclude<Keys, K>>>;
}[Keys];
export type RequireOnlyOne<T, Keys extends keyof T = keyof T> = Pick<T, Exclude<keyof T, Keys>> & {
    [K in Keys]-?: Required<Pick<T, K>> & Partial<Record<Exclude<Keys, K>, undefined>>;
}[Keys];
export type KeysOfUnion<T> = T extends T ? keyof T : never;
type UndefinedProperties<T> = {
    [P in keyof T]-?: undefined extends T[P] ? P : never;
}[keyof T];
type HasNoRequiredParameters<X> = RequiredKeys<X> extends never ? true : false;
export type ExcludeKeysWhereHasNoRequiredParametersTrue<T> = {
    [K in keyof T]: HasNoRequiredParameters<T[K]> extends true ? never : K;
}[keyof T];
export type ExcludeKeysWhereHasNoRequiredParametersFalse<T> = {
    [K in keyof T]: HasNoRequiredParameters<T[K]> extends false ? never : K;
}[keyof T];
type ExcludeEmpty<X> = {
    [K in ExcludeKeysWhereHasNoRequiredParametersTrue<X>]: X[K];
};
type OnlyEmpty<X> = {
    [K in ExcludeKeysWhereHasNoRequiredParametersFalse<X>]: X[K];
};
export type EmptyOptional<X> = ExcludeEmpty<X> & Optional<OnlyEmpty<X>>;
export type ToOptional<T> = Partial<Pick<T, UndefinedProperties<T>>> & Pick<T, Exclude<keyof T, UndefinedProperties<T>>>;
export type KeysWithValueType<Obj, Type> = {
    [Prop in keyof Obj]: Obj[Prop] extends Type ? Prop : never;
}[keyof Obj];
export type KeysOfType<Obj, Type> = {
    [Prop in keyof Obj]: Prop extends Type ? Prop : never;
}[keyof Obj];
export type StringKeys<Obj> = KeysOfType<Obj, string>;
export type ThenArg<T> = T extends PromiseLike<infer U> ? ThenArg<U> : T;
export type RemoveStringPrefix<T extends string, Prefix extends string> = T extends `${Prefix}${infer Postfix}` ? Postfix : never;
export interface IConstruct<T extends new (...args: any) => any> {
    type: new (...args: ConstructorParameters<T>) => InstanceType<T>;
}
export type TFn = (...params: any[]) => any;
export type TAsyncFn<RT = any> = (...params: any[]) => Promise<RT>;
export type RequiredKeys<T> = {
    [K in keyof T]-?: {} extends Pick<T, K> ? never : K;
}[keyof T];
export type OptionalKeys<T> = {
    [K in keyof T]-?: {} extends Pick<T, K> ? K : never;
}[keyof T];
type InitDepth = 3;
type Prev = [never, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, ...0[]];
type StringLiteral<T> = T extends string ? string extends T ? never : T : never;
type Join<K, P> = K extends string | number ? P extends string | number ? `${K}${'' extends StringLiteral<P> ? '' : '.'}${P}` : never : never;
type Paths<T, D extends number = InitDepth> = [D] extends [never] ? never : T extends object ? {
    [K in keyof T]-?: K extends string | number ? `${K}` | Join<K, Paths<T[K], Prev[D]>> : never;
}[keyof T] : '';
type Leaves<T, D extends number = InitDepth> = [D] extends [never] ? never : T extends object ? {
    [K in keyof T]-?: Join<K, Leaves<T[K], Prev[D]>>;
}[keyof T] : '';
export type NestedObjectPaths<Obj extends object> = Paths<Obj>;
export type NestedObjectLeaves<Obj extends object> = Leaves<Obj>;
export type TDottedRecord<Obj extends object> = {
    [K in NestedObjectPaths<Obj>]: Object.Path<Obj, String.Split<K, '.'>>;
};
export type Util_FlatType<T> = T extends object ? {
    [K in keyof T]: Util_FlatType<T[K]>;
} : T;
type DeepMergeHead<T> = T extends [infer I, ...infer _Rest] ? I : never;
type DeepMergeTail<T> = T extends [infer _I, ...infer Rest] ? Rest : never;
type Zip_DeepMergeTwoTypes<T, U> = T extends [] ? U : U extends [] ? T : [
    DeepMergeTwoTypes<DeepMergeHead<T>, DeepMergeHead<U>>,
    ...Zip_DeepMergeTwoTypes<DeepMergeTail<T>, DeepMergeTail<U>>
];
/**
 * Take two objects T and U and create the new one with uniq keys for T a U objectI
 * helper generic for `DeepMergeTwoTypes`
 */
type DeepMergeGetObjDifferentKeys<T, U, T0 = Omit<T, keyof U> & Omit<U, keyof T>, T1 = {
    [K in keyof T0]: T0[K];
}> = T1;
/**
 * Take two objects T and U and create the new one with the same objects keys
 * helper generic for `DeepMergeTwoTypes`
 */
type DeepMergeGetObjSameKeys<T, U> = Omit<T | U, keyof DeepMergeGetObjDifferentKeys<T, U>>;
type DeepMergeMergeTwoObjects<T, U, T0 = Partial<DeepMergeGetObjDifferentKeys<T, U>> & {
    [K in keyof DeepMergeGetObjSameKeys<T, U>]: DeepMergeTwoTypes<T[K], U[K]>;
}, T1 = {
    [K in keyof T0]: T0[K];
}> = T1;
export type DeepMergeTwoTypes<T, U> = [
    T,
    U
] extends [any[], any[]] ? Zip_DeepMergeTwoTypes<T, U> : [T, U] extends [{
    [key: string]: unknown;
}, {
    [key: string]: unknown;
}] ? DeepMergeMergeTwoObjects<T, U> : T | U;
type Previous = [never, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, ...0[]];
export type PathsFast<TEntity, TKey extends number | string = string, TDepth extends number = 20> = [TDepth] extends [
    never
] ? never : TEntity extends object ? {
    [Key in keyof TEntity]-?: Key extends TKey ? `${Key}` | Join<Key, Paths<TEntity[Key], Previous[TDepth]>> : never;
}[keyof TEntity] : '';
export type TypeAtPathArr<T extends object, Path extends string[]> = Path extends [infer Head, ...infer Tail] ? Head extends keyof T ? Tail extends string[] ? Tail extends [] ? T[Head] : T[Head] extends object ? TypeAtPathArr<T[Head], Tail> : never : never : never : never;
export type TypeAtPath<T extends object, Path extends string> = TypeAtPathArr<T, String.Split<Path, '.'>>;
type HasAllKeysIn_Asymmetrical<A extends object, B extends object> = {
    [K in keyof A]: K extends keyof B ? A[K] extends B[K] ? true : B[K] extends A[K] ? true : false : false;
};
export type HasAllKeysIn<A extends object, B extends object> = (HasAllKeysIn_Asymmetrical<B, A>) extends Record<string, true> ? true : false;
export type If<Condition extends boolean, Object extends {}> = Condition extends true ? Object : {};
export type Not<Condition extends boolean> = Condition extends true ? false : true;
export type And<C1 extends boolean, C2 extends boolean> = C1 extends true ? C2 extends true ? true : false : false;
export type StringDoentContain<S extends string, Substring extends string> = String.Replace<S, Substring, '___SHOULD_NOT_APPEAR_IN_RESULT___'> extends S ? true : false;
export type RemoveLast<List extends any[]> = List extends [infer A, ...infer Rest] ? Rest extends [] ? [] : [A, ...RemoveLast<Rest>] : [];
export type OmitNever<T> = {
    [K in keyof T as T[K] extends never ? never : K]: T[K];
};
export {};
